# ostruct_r.rb  (encoding: utf-8)  2015/06/09 by T. Yoshiizumi
require "ostruct"

class OpenStruct_r < OpenStruct
  @@ost_temp_obj = OpenStruct.new

  def ost_fl_ren(str)  # 'abc' -> 'Abc' or 'Abc' -> 'abc'
    if str =~ /^([a-z])(.+)$/i
      first_l = $1
      rest = $2
      first_l = (first_l =~ /[A-Z]/) ? first_l.downcase : first_l.upcase
      str = first_l + rest
    end
    return str
  end
  private :ost_fl_ren

  def ost_prd_split(key, val)  # 'a.b.c' -> {'a'=>{'b'=>{'c'=>xxx}}}
    hs = {}
    key_ary = key.split(/\./)
    while key_ary.size > 1
      hs = (hs.size == 0) ? {key_ary.pop => val} :
          {key_ary.pop => hs}
    end
    val = hs  if hs.size > 0
    return [key_ary[0], val]
  end
  private :ost_prd_split

  # child Hash -> OpenStruct_r object
  # ' ', '+', '-' and so on (in Hash's key) -> '_'
  def ost_hash_conv(x)
    reg = /[ "#%&'()*+,\-\/:;<=>?\[\\\]^`{|}~]/
    if x.class == Hash
      x2 = {}
      x.each do |key, val|
        key = key.to_s.gsub(reg, "_")
        if key =~ /\./
          key, val = ost_prd_split(key, val)
        end
        if val.class == Hash
          val = OpenStruct_r.new(val)
        end
        bval = x2[key]
        if [OpenStruct, OpenStruct_r].include?(bval.class)
          val.each_pair do |kk, vv|
            bval[kk] = vv
          end
          val = bval
        end
        if @@ost_temp_obj.respond_to?(key, true) or
          %w(ost_fl_ren ost_prd_split ost_hash_conv ost_all all).include?(key)
          key = ost_fl_ren(key)
        end
        x2[key] = val
      end
      x = x2
    end
    return x
  end
  private :ost_hash_conv

  # gets an OpenStruct_r object's all pairs (element and value)
  # ex.: returns [["xx.statistic.W", 0.0], "xx.parameter", nil], ...]
  def ost_all(xx, xname)
    res = []
    xx.each_pair do |key, val|
      if [OpenStruct, OpenStruct_r].include?(val.class)
        ary = ost_all(val, "#{xname}.#{key}")
        res += ary
      else
        res << ["#{xname}.#{key}", val]
      end
    end
    return res
  end
  private :ost_all

  def initialize(arg = nil)
    arg = ost_hash_conv(arg)
    super
  end

  def all(xname = "xx", &block)
    res = ost_all(self, xname)
    if block
      res.each {|element|  block.call(element)}
      return
    end
    return res
  end
end
